# -*- coding: utf-8 -*-
from django.conf import settings  # NOQA

from appconf import AppConf

__all__ = ["settings"]


class CookieConsentConf(AppConf):
    # django-cookie-consent cookie settings that store the configuration
    NAME = "cookie_consent"
    # TODO: rename to AGE for parity with django settings
    MAX_AGE = 60 * 60 * 24 * 365 * 1  # 1 year,
    DOMAIN = None
    SECURE = False
    HTTPONLY = True
    SAMESITE = "Lax"

    DECLINE = "-1"

    ENABLED = True

    OPT_OUT = False

    CACHE_BACKEND = "default"

    LOG_ENABLED = True



# -*- coding: utf-8 -*-
from django.test import TestCase
from django.test.utils import override_settings
from django.urls import reverse

from cookie_consent.models import (
    ACTION_ACCEPTED,
    ACTION_DECLINED,
    Cookie,
    CookieGroup,
    LogItem,
)


class CookieGroupBaseProcessViewTests(TestCase):
    def test_get_success_url(self):
        """
        If user adds a 'next' as URL parameter it should,
        redirect to the value of 'next'
        """
        expected_url = reverse("test_page")
        url = "{}?next={}".format(reverse("cookie_consent_accept_all"), expected_url)
        response = self.client.post(url, follow=True)
        self.assertRedirects(response, expected_url)

    def test_no_open_redirects(self):
        url = "{}?next=https://evil.com".format(reverse("cookie_consent_accept_all"))
        response = self.client.post(url, follow=True)
        self.assertEqual(response.status_code, 400)  # result of SupiciousOperation


class IntegrationTest(TestCase):
    def setUp(self):
        self.cookie_group = CookieGroup.objects.create(
            varname="optional",
            name="Optional",
        )
        self.cookie = Cookie.objects.create(
            cookiegroup=self.cookie_group,
            name="foo",
        )

    def test_cookies_view(self):
        response = self.client.get(reverse("cookie_consent_cookie_group_list"))
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, '<input type="submit" value="Accept">')
        self.assertContains(response, '<input type="submit" value="Decline">')

    def test_accept_cookie(self):
        response = self.client.post(
            reverse("cookie_consent_accept", kwargs={"varname": "optional"}),
            follow=True,
        )
        self.assertEqual(response.status_code, 200)
        self.assertContains(
            response, '<span class="cookie-consent-accepted">Accepted</span>'
        )

    def test_accept_cookie_ajax(self):
        response = self.client.post(
            reverse("cookie_consent_accept", kwargs={"varname": "optional"}),
            HTTP_X_REQUESTED_WITH="XMLHttpRequest",
        )
        self.assertEqual(response.status_code, 200)

    def test_decline_cookie(self):
        response = self.client.post(
            reverse("cookie_consent_decline", kwargs={"varname": "optional"}),
            follow=True,
        )
        self.assertEqual(response.status_code, 200)
        self.assertContains(
            response, '<span class="cookie-consent-declined">Declined</span>'
        )

    def test_decline_cookie_ajax(self):
        response = self.client.delete(
            reverse("cookie_consent_decline", kwargs={"varname": "optional"}),
            HTTP_X_REQUESTED_WITH="XMLHttpRequest",
        )
        self.assertEqual(response.status_code, 200)

    def test_cookies(self):
        CookieGroup.objects.create(
            varname="social",
            name="Social",
        )
        Cookie.objects.create(
            cookiegroup=self.cookie_group,
            name="optional_test_cookie",
        )

        url = reverse("test_page")
        response = self.client.get(url)
        self.assertContains(response, '"optional" cookies not accepted or declined')

        response = self.client.post(
            reverse("cookie_consent_accept", kwargs={"varname": "optional"})
        )
        response = self.client.get(url)
        self.assertContains(response, '"optional" cookies accepted')
        self.assertEqual(
            self.client.cookies.get("optional_test_cookie").value,
            "optional cookie set from django",
        )

        response = self.client.post(
            reverse("cookie_consent_decline", kwargs={"varname": "optional"})
        )
        response = self.client.get(url)
        self.assertContains(response, '"optional" cookies declined')
        # test client returns cookie with value of ''
        # self.assertIsNone(self.client.cookies.get('optional_test_cookie'))
        self.assertFalse(self.client.cookies.get("optional_test_cookie").value)

    def test_logging(self):
        self.client.post(
            reverse("cookie_consent_accept", kwargs={"varname": "optional"})
        )
        log_items = LogItem.objects.filter(
            cookiegroup=self.cookie_group,
            version=self.cookie_group.get_version(),
            action=ACTION_ACCEPTED,
        )
        self.assertEqual(log_items.count(), 1)

        self.client.delete(
            reverse("cookie_consent_decline", kwargs={"varname": "optional"})
        )
        log_items = LogItem.objects.filter(
            cookiegroup=self.cookie_group,
            version=self.cookie_group.get_version(),
            action=ACTION_DECLINED,
        )
        self.assertEqual(log_items.count(), 1)

    @override_settings(COOKIE_CONSENT_LOG_ENABLED=False)
    def test_logging_disabled(self):
        """
        When the COOKIE_CONSENT_LOG_ENABLED is False, no log item should be
        created
        """
        self.client.post(
            reverse("cookie_consent_accept", kwargs={"varname": "optional"})
        )
        log_items = LogItem.objects.filter(
            cookiegroup=self.cookie_group,
            version=self.cookie_group.get_version(),
            action=ACTION_ACCEPTED,
        )
        self.assertEqual(log_items.count(), 0)

        self.client.delete(
            reverse("cookie_consent_decline", kwargs={"varname": "optional"})
        )
        log_items = LogItem.objects.filter(
            cookiegroup=self.cookie_group,
            version=self.cookie_group.get_version(),
            action=ACTION_DECLINED,
        )
        self.assertEqual(log_items.count(), 0)



import pytest

from cookie_consent.models import CookieGroup


def test_natural_key():
    group = CookieGroup(varname="social")

    assert group.natural_key() == ("social",)


@pytest.mark.django_db
def test_load_by_natural_key():
    social_group = CookieGroup.objects.create(varname="social")
    CookieGroup.objects.create(varname="other")

    loaded_group = CookieGroup.objects.get_by_natural_key("social")

    assert loaded_group == social_group



import pytest

from cookie_consent.models import Cookie, CookieGroup


def test_natural_key():
    cookie = Cookie(
        cookiegroup=CookieGroup(varname="analytics"), name="trck", domain="example.com"
    )

    assert cookie.natural_key() == ("trck", "example.com", "analytics")


@pytest.mark.django_db
def test_load_by_natural_key():
    social_group = CookieGroup.objects.create(varname="social")
    cookie = Cookie.objects.create(
        cookiegroup=social_group, name="trck", domain="example.com"
    )
    Cookie.objects.create(cookiegroup=social_group, name="other", domain="example.com")

    loaded_cookie = Cookie.objects.get_by_natural_key("trck", "example.com", "social")

    assert loaded_cookie == cookie



# Generated by Django 2.1 on 2019-02-08 14:16

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("cookie_consent", "0001_initial"),
    ]

    operations = [
        migrations.CreateModel(
            name="LogItem",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "action",
                    models.IntegerField(
                        choices=[(-1, "Declined"), (1, "Accepted")],
                        verbose_name="Action",
                    ),
                ),
                ("version", models.CharField(max_length=32, verbose_name="Version")),
                (
                    "created",
                    models.DateTimeField(auto_now_add=True, verbose_name="Created"),
                ),
                (
                    "cookiegroup",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="cookie_consent.CookieGroup",
                        verbose_name="Cookie Group",
                    ),
                ),
            ],
            options={
                "verbose_name": "Log item",
                "verbose_name_plural": "Log items",
                "ordering": ["-created"],
            },
        ),
    ]



# -*- coding: utf-8 -*-
from django.contrib.auth.views import RedirectURLMixin
from django.core.exceptions import SuspiciousOperation
from django.http import HttpRequest, HttpResponse, HttpResponseRedirect, JsonResponse
from django.middleware.csrf import get_token as get_csrf_token
from django.urls import reverse
from django.utils.http import url_has_allowed_host_and_scheme
from django.views.generic import ListView, View

from .models import CookieGroup
from .util import (
    accept_cookies,
    decline_cookies,
    get_accepted_cookie_groups,
    get_declined_cookie_groups,
    get_not_accepted_or_declined_cookie_groups,
)


def is_ajax_like(request: HttpRequest) -> bool:
    # legacy ajax, removed in Django 4.0 (used to be request.is_ajax())
    ajax_header = request.headers.get("X-Requested-With")
    if ajax_header == "XMLHttpRequest":
        return True

    # module-js uses fetch and a custom header
    return bool(request.headers.get("X-Cookie-Consent-Fetch"))


class CookieGroupListView(ListView):
    """
    Display all cookies.
    """

    model = CookieGroup


class CookieGroupBaseProcessView(RedirectURLMixin, View):
    def get_success_url(self):
        """
        If user adds a 'next' as URL parameter or hidden input,
        redirect to the value of 'next'. Otherwise, redirect to
        cookie consent group list
        """
        redirect_to = self.request.POST.get("next", self.request.GET.get("next"))
        if redirect_to and not url_has_allowed_host_and_scheme(
            url=redirect_to,
            allowed_hosts=self.get_success_url_allowed_hosts(),
            require_https=self.request.is_secure(),
        ):
            raise SuspiciousOperation("Unsafe open redirect suspected.")
        return redirect_to or reverse("cookie_consent_cookie_group_list")

    def process(self, request, response, varname):  # pragma: no cover
        raise NotImplementedError()

    def post(self, request, *args, **kwargs):
        varname = kwargs.get("varname", None)
        if is_ajax_like(request):
            response = HttpResponse()
        else:
            response = HttpResponseRedirect(self.get_success_url())
        self.process(request, response, varname)
        return response


class CookieGroupAcceptView(CookieGroupBaseProcessView):
    """
    View to accept CookieGroup.
    """

    def process(self, request, response, varname):
        accept_cookies(request, response, varname)


class CookieGroupDeclineView(CookieGroupBaseProcessView):
    """
    View to decline CookieGroup.
    """

    def process(self, request, response, varname):
        decline_cookies(request, response, varname)

    def delete(self, request, *args, **kwargs):
        return self.post(request, *args, **kwargs)


class CookieStatusView(View):
    """
    Check the current accept/decline status for cookies.

    The returned accept and decline URLs are specific to this user and include the
    cookie groups that weren't accepted or declined yet.

    Note that this endpoint also returns a CSRF Token to be used by the frontend,
    as baking a CSRFToken into a cached page will not reliably work.
    """

    def get(self, request: HttpRequest) -> JsonResponse:
        accepted = get_accepted_cookie_groups(request)
        declined = get_declined_cookie_groups(request)
        not_accepted_or_declined = get_not_accepted_or_declined_cookie_groups(request)
        # TODO: change this csv URL param into proper POST params
        varnames = ",".join([group.varname for group in not_accepted_or_declined])
        data = {
            "csrftoken": get_csrf_token(request),
            "acceptUrl": reverse("cookie_consent_accept", kwargs={"varname": varnames}),
            "declineUrl": reverse(
                "cookie_consent_decline", kwargs={"varname": varnames}
            ),
            "acceptedCookieGroups": [group.varname for group in accepted],
            "declinedCookieGroups": [group.varname for group in declined],
            "notAcceptedOrDeclinedCookieGroups": [
                group.varname for group in not_accepted_or_declined
            ],
        }
        return JsonResponse(data)



# -*- coding: utf-8 -*-
from django.urls import path, re_path
from django.views.decorators.csrf import csrf_exempt

from .views import (
    CookieGroupAcceptView,
    CookieGroupDeclineView,
    CookieGroupListView,
    CookieStatusView,
)

urlpatterns = [
    path(
        "accept/",
        csrf_exempt(CookieGroupAcceptView.as_view()),
        name="cookie_consent_accept_all",
    ),
    # TODO: use form or query string params for this instead?
    re_path(
        r"^accept/(?P<varname>.*)/$",
        csrf_exempt(CookieGroupAcceptView.as_view()),
        name="cookie_consent_accept",
    ),
    # TODO: use form or query string params for this instead?
    re_path(
        r"^decline/(?P<varname>.*)/$",
        csrf_exempt(CookieGroupDeclineView.as_view()),
        name="cookie_consent_decline",
    ),
    path(
        "decline/",
        csrf_exempt(CookieGroupDeclineView.as_view()),
        name="cookie_consent_decline_all",
    ),
    path("status/", CookieStatusView.as_view(), name="cookie_consent_status"),
    path("", CookieGroupListView.as_view(), name="cookie_consent_cookie_group_list"),
]



from django.urls import reverse

import pytest

pytestmark = pytest.mark.django_db


@pytest.mark.parametrize(
    ("setting", "value", "assertion"),
    [
        ("MAX_AGE", 3600, {"max-age": 3600}),
        ("DOMAIN", None, {"domain": ""}),
        ("DOMAIN", "example.com", {"domain": "example.com"}),
        ("SECURE", True, {"secure": True}),
        ("SECURE", False, {"secure": ""}),
        ("SECURE", None, {"secure": ""}),
        ("HTTPONLY", True, {"httponly": True}),
        ("HTTPONLY", None, {"httponly": ""}),
        ("HTTPONLY", False, {"httponly": ""}),
        ("SAMESITE", "Lax", {"samesite": "Lax"}),
        ("SAMESITE", None, {"samesite": ""}),
        ("SAMESITE", False, {"samesite": ""}),
        ("SAMESITE", "None", {"samesite": "None"}),
        ("SAMESITE", "Strict", {"samesite": "Strict"}),
    ],
)
def test_cookie_consent_cookie_options(settings, client, setting, value, assertion):
    accept_all_url = reverse("cookie_consent_accept_all")
    setattr(settings, f"COOKIE_CONSENT_{setting}", value)

    client.post(accept_all_url)

    cookie = client.cookies[settings.COOKIE_CONSENT_NAME]
    for key, expected in assertion.items():
        assert cookie[key] == expected



# Generated by Django 2.1 on 2019-02-08 14:14

import re

import django.core.validators
import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):
    initial = True

    dependencies = []

    operations = [
        migrations.CreateModel(
            name="Cookie",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("name", models.CharField(max_length=250, verbose_name="Name")),
                (
                    "description",
                    models.TextField(blank=True, verbose_name="Description"),
                ),
                (
                    "path",
                    models.TextField(blank=True, default="/", verbose_name="Path"),
                ),
                (
                    "domain",
                    models.CharField(blank=True, max_length=250, verbose_name="Domain"),
                ),
                (
                    "created",
                    models.DateTimeField(auto_now_add=True, verbose_name="Created"),
                ),
            ],
            options={
                "verbose_name": "Cookie",
                "verbose_name_plural": "Cookies",
                "ordering": ["-created"],
            },
        ),
        migrations.CreateModel(
            name="CookieGroup",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "varname",
                    models.CharField(
                        max_length=32,
                        validators=[
                            django.core.validators.RegexValidator(
                                re.compile("^[-_a-zA-Z0-9]+$"),
                                "Enter a valid 'varname' consisting of letters, numbers, underscores or hyphens.",
                                "invalid",
                            )
                        ],
                        verbose_name="Variable name",
                    ),
                ),
                (
                    "name",
                    models.CharField(blank=True, max_length=100, verbose_name="Name"),
                ),
                (
                    "description",
                    models.TextField(blank=True, verbose_name="Description"),
                ),
                (
                    "is_required",
                    models.BooleanField(
                        default=False,
                        help_text="Are cookies in this group required.",
                        verbose_name="Is required",
                    ),
                ),
                (
                    "is_deletable",
                    models.BooleanField(
                        default=True,
                        help_text="Can cookies in this group be deleted.",
                        verbose_name="Is deletable?",
                    ),
                ),
                ("ordering", models.IntegerField(default=0, verbose_name="Ordering")),
                (
                    "created",
                    models.DateTimeField(auto_now_add=True, verbose_name="Created"),
                ),
            ],
            options={
                "verbose_name": "Cookie Group",
                "verbose_name_plural": "Cookie Groups",
                "ordering": ["ordering"],
            },
        ),
        migrations.AddField(
            model_name="cookie",
            name="cookiegroup",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                to="cookie_consent.CookieGroup",
                verbose_name="Cookie Group",
            ),
        ),
    ]



# Generated by Django 4.2.13 on 2024-05-09 19:01

import re

import django.core.validators
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("cookie_consent", "0002_auto__add_logitem"),
    ]

    operations = [
        migrations.AlterField(
            model_name="cookiegroup",
            name="varname",
            field=models.CharField(
                max_length=32,
                unique=True,
                validators=[
                    django.core.validators.RegexValidator(
                        re.compile("^[-_a-zA-Z0-9]+$"),
                        "Enter a valid 'varname' consisting of letters, numbers, underscores or hyphens.",
                        "invalid",
                    )
                ],
                verbose_name="Variable name",
            ),
        ),
    ]



# -*- coding: utf-8 -*-
from django.core.cache import caches

from .conf import settings
from .models import CookieGroup

CACHE_KEY = "cookie_consent_cache"
CACHE_TIMEOUT = 60 * 60  # 60 minutes


def _get_cache():
    """
    Lazily wrap around django.core.cache.

    This prevents the cache object to be resolved at import-time, which breaks the
    `django.test.override_settings` functionality for projects adding tests for this
    package, see https://github.com/bmihelac/django-cookie-consent/issues/41.
    """
    return caches[settings.COOKIE_CONSENT_CACHE_BACKEND]


def delete_cache():
    cache = _get_cache()
    cache.delete(CACHE_KEY)


def _get_cookie_groups_from_db():
    qs = CookieGroup.objects.filter(is_required=False).prefetch_related("cookie_set")
    return qs.in_bulk(field_name="varname")


def all_cookie_groups():
    """
    Get all cookie groups that are optional.

    Reads from the cache where possible, sets the value in the cache if there's a
    cache miss.
    """
    cache = _get_cache()
    return cache.get_or_set(
        CACHE_KEY, _get_cookie_groups_from_db, timeout=CACHE_TIMEOUT
    )


def get_cookie_group(varname):
    return all_cookie_groups().get(varname)


def get_cookie(cookie_group, name, domain):
    for cookie in cookie_group.cookie_set.all():
        if cookie.name == name and cookie.domain == domain:
            return cookie
    return None



"""
Test the behaviour of the dynamic (JS based) cookiebar module.

See docs: https://playwright.dev/python/docs/test-runners for CLI options.
"""

from django.urls import reverse

import pytest
from playwright.sync_api import Page, expect

pytestmark = [pytest.mark.django_db, pytest.mark.e2e]


COOKIE_BAR_CONTENT = """
This site uses Social, Optional cookies for better performance and user experience.
Do you agree to use these cookies?
"""


@pytest.fixture(scope="function", autouse=True)
def before_each_after_each(live_server, page: Page, load_testapp_fixture):
    test_page_url = f"{live_server.url}{reverse('test_page')}"
    page.goto(test_page_url)
    marker = page.get_by_text("page-done-loading")
    expect(marker).to_be_visible()
    yield


def test_cookiebar_shows_initially(page: Page):
    cookiebar = page.get_by_text(COOKIE_BAR_CONTENT)
    expect(cookiebar).to_be_visible()


def test_cookiebar_accept_all(page: Page):
    accept_button = page.get_by_role("button", name="Accept")
    expect(accept_button).to_be_visible()

    accept_button.click()

    expect(page.get_by_text(COOKIE_BAR_CONTENT)).not_to_be_visible()
    share_button = page.get_by_role("button", name="SHARE")
    expect(share_button).to_be_visible()


def test_cookiebar_decline_all(page: Page):
    decline_button = page.get_by_role("button", name="Decline")
    expect(decline_button).to_be_visible()

    decline_button.click()

    expect(page.get_by_text(COOKIE_BAR_CONTENT)).not_to_be_visible()
    share_button = page.get_by_role("button", name="SHARE")
    expect(share_button).not_to_be_visible()


@pytest.mark.parametrize("btn_text", ["Accept", "Decline"])
def test_cookiebar_not_shown_anymore_after_accept_or_decline(btn_text: str, page: Page):
    expect(page.get_by_text(COOKIE_BAR_CONTENT)).to_be_visible()

    button = page.get_by_role("button", name=btn_text)
    expect(button).to_be_visible()

    button.click()
    expect(page.get_by_text(COOKIE_BAR_CONTENT)).not_to_be_visible()

    page.reload()
    expect(page.get_by_text(COOKIE_BAR_CONTENT)).not_to_be_visible()


def test_on_accept_handler_runs_on_load(page: Page, live_server):
    accept_button = page.get_by_role("button", name="Accept")
    accept_button.click()
    page.wait_for_load_state("networkidle")
    # wait for fetch calls to complete & avoid test race conditions...
    share_button = page.get_by_role("button", name="SHARE")
    expect(share_button).to_be_visible()

    test_page_url = f"{live_server.url}{reverse('test_page')}"
    page.goto(test_page_url)
    marker = page.get_by_text("page-done-loading")
    expect(marker).to_be_visible()

    share_button = page.get_by_role("button", name="SHARE")
    expect(share_button).to_be_visible()



# Generated by Django 4.2.13 on 2024-05-09 20:22

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("cookie_consent", "0003_alter_cookiegroup_varname"),
    ]

    operations = [
        migrations.AddConstraint(
            model_name="cookie",
            constraint=models.UniqueConstraint(
                fields=("cookiegroup", "name", "domain"), name="natural_key"
            ),
        ),
    ]



# -*- coding: utf-8 -*-
from copy import deepcopy

from django.conf import settings
from django.core.cache import caches
from django.core.exceptions import ValidationError
from django.test import TestCase, override_settings

from cookie_consent.cache import CACHE_KEY, delete_cache
from cookie_consent.models import Cookie, CookieGroup, validate_cookie_name

patch_caches = override_settings(
    COOKIE_CONSENT_CACHE_BACKEND="tests",
    CACHES={
        **deepcopy(settings.CACHES),
        "tests": {
            "BACKEND": "django.core.cache.backends.locmem.LocMemCache",
        },
    },
)


class CacheMixin:
    def populateCache(self):
        cache = caches["tests"]
        cache.set(CACHE_KEY, {}, timeout=3600)

    def assertCacheNotPopulated(self):
        cache = caches["tests"]
        has_key = cache.has_key(CACHE_KEY)
        self.assertFalse(has_key)


@patch_caches
class CookieGroupTest(CacheMixin, TestCase):
    def setUp(self):
        self.addCleanup(delete_cache)

        self.cookie_group = CookieGroup.objects.create(
            varname="optional",
            name="Optional",
        )
        self.cookie = Cookie.objects.create(
            cookiegroup=self.cookie_group,
            name="foo",
        )

    def test_get_version(self):
        self.assertEqual(
            self.cookie_group.get_version(), self.cookie.created.isoformat()
        )

    def test_bulk_delete(self):
        self.populateCache()

        deleted_objs_count, _ = CookieGroup.objects.filter(
            id=self.cookie_group.id
        ).delete()

        # Deleting a CookieGroup also deletes the associated Cookies, that's why we expect a count of 2.
        self.assertEqual(deleted_objs_count, 2)
        self.assertCacheNotPopulated()

    def test_bulk_update(self):
        self.populateCache()

        updated_objs_count = CookieGroup.objects.filter(id=self.cookie_group.id).update(
            name="Optional2"
        )

        self.assertEqual(updated_objs_count, 1)
        self.assertCacheNotPopulated()


@patch_caches
class CookieTest(CacheMixin, TestCase):
    def setUp(self):
        self.addCleanup(delete_cache)

        self.cookie_group = CookieGroup.objects.create(
            varname="optional",
            name="Optional",
        )
        self.cookie = Cookie.objects.create(
            cookiegroup=self.cookie_group,
            name="foo",
            domain=".example.com",
        )

    def test_varname(self):
        self.assertEqual(self.cookie.varname, "optional=foo:.example.com")

    def test_bulk_delete(self):
        self.populateCache()

        deleted_objs_count, _ = Cookie.objects.filter(id=self.cookie.id).delete()

        self.assertEqual(deleted_objs_count, 1)
        self.assertCacheNotPopulated()

    def test_bulk_update(self):
        self.populateCache()

        updated_objs_count = Cookie.objects.filter(id=self.cookie.id).update(
            name="foo2"
        )

        self.assertEqual(updated_objs_count, 1)
        self.assertCacheNotPopulated()


class ValidateCookieNameTest(TestCase):
    def test_valid(self):
        validate_cookie_name("_foo-bar")

    def test_invalid(self):
        invalid_names = (
            "space inside",
            "a!b",
            "$",
        )
        for name in invalid_names:
            with self.assertRaises(ValidationError):
                validate_cookie_name("no spaces")
